{% if strengths.result |length > 0 %}
# Strengths

{{ strengths.result }}

---

{% endif %}
{% if weaknesses.result |length > 0 %}
# Weaknesses

{{ weaknesses.result }}

---

{% endif %}
{% if opportunities.result |length > 0 %}
# Opportunities

{{ opportunities.result }}

---

{% endif %}
{% if threats.result |length > 0 %}
# Threats

{{ threats.result }}

---

{% endif %}